@@ -53,7 +53,6 @@ class BacktestingEngine:
         self.risk_free: float = 0
         self.annual_days: int = 240
         self.mode: BacktestingMode = BacktestingMode.BAR
-        self.inverse: bool = False
 
         self.strategy_class: Type[CtaTemplate] = None
         self.strategy: CtaTemplate = None
@@ -117,7 +116,6 @@ class BacktestingEngine:
         capital: int = 0,
         end: datetime = None,
         mode: BacktestingMode = BacktestingMode.BAR,
-        inverse: bool = False,
         risk_free: float = 0,
         annual_days: int = 240
     ) -> None:
@@ -137,7 +135,6 @@ class BacktestingEngine:
         self.capital = capital
         self.end = end
         self.mode = mode
-        self.inverse = inverse
         self.risk_free = risk_free
         self.annual_days = annual_days
 
@@ -288,8 +285,7 @@ class BacktestingEngine:
                 start_pos,
                 self.size,
                 self.rate,
-                self.slippage,
-                self.inverse
+                self.slippage
             )
 
             pre_close = daily_result.close_price
@@ -315,35 +311,37 @@ class BacktestingEngine:
         if df is None:
             df: DataFrame = self.daily_df
 
-        # Check for init DataFrame
-        if df is None:
-            # Set all statistics to 0 if no trade.
-            start_date: str = ""
-            end_date: str = ""
-            total_days: int = 0
-            profit_days: int = 0
-            loss_days: int = 0
-            end_balance: float = 0
-            max_drawdown: float = 0
-            max_ddpercent: float = 0
-            max_drawdown_duration: int = 0
-            total_net_pnl: float = 0
-            daily_net_pnl: float = 0
-            total_commission: float = 0
-            daily_commission: float = 0
-            total_slippage: float = 0
-            daily_slippage: float = 0
-            total_turnover: float = 0
-            daily_turnover: float = 0
-            total_trade_count: int = 0
-            daily_trade_count: int = 0
-            total_return: float = 0
-            annual_return: float = 0
-            daily_return: float = 0
-            return_std: float = 0
-            sharpe_ratio: float = 0
-            return_drawdown_ratio: float = 0
-        else:
+        # Init all statistics default value
+        start_date: str = ""
+        end_date: str = ""
+        total_days: int = 0
+        profit_days: int = 0
+        loss_days: int = 0
+        end_balance: float = 0
+        max_drawdown: float = 0
+        max_ddpercent: float = 0
+        max_drawdown_duration: int = 0
+        total_net_pnl: float = 0
+        daily_net_pnl: float = 0
+        total_commission: float = 0
+        daily_commission: float = 0
+        total_slippage: float = 0
+        daily_slippage: float = 0
+        total_turnover: float = 0
+        daily_turnover: float = 0
+        total_trade_count: int = 0
+        daily_trade_count: int = 0
+        total_return: float = 0
+        annual_return: float = 0
+        daily_return: float = 0
+        return_std: float = 0
+        sharpe_ratio: float = 0
+        return_drawdown_ratio: float = 0
+
+        # Check if balance is always positive
+        positive_balance: bool = False
+
+        if df is not None:
             # Calculate balance related time series data
             df["balance"] = df["net_pnl"].cumsum() + self.capital
 
@@ -361,6 +359,13 @@ class BacktestingEngine:
             df["drawdown"] = df["balance"] - df["highlevel"]
             df["ddpercent"] = df["drawdown"] / df["highlevel"] * 100
 
+            # All balance value needs to be positive
+            positive_balance = (df["balance"] > 0).all()
+            if not positive_balance:
+                self.output("回测中出现爆仓（资金小于等于0），无法计算策略统计指标")
+
+        # Calculate statistics value
+        if positive_balance:
             # Calculate statistics value
             start_date = df.index[0]
             end_date = df.index[-1]
@@ -406,7 +411,10 @@ class BacktestingEngine:
             else:
                 sharpe_ratio: float = 0
 
-            return_drawdown_ratio: float = -total_return / max_ddpercent
+            if max_ddpercent:
+                return_drawdown_ratio: float = -total_return / max_ddpercent
+            else:
+                return_drawdown_ratio = 0
 
         # Output
         if output:
@@ -525,7 +533,12 @@ class BacktestingEngine:
         fig.update_layout(height=1000, width=1000)
         fig.show()
 
-    def run_bf_optimization(self, optimization_setting: OptimizationSetting, output=True) -> list:
+    def run_bf_optimization(
+        self,
+        optimization_setting: OptimizationSetting,
+        output: bool = True,
+        max_workers: int = None
+    ) -> list:
         """"""
         if not check_optimization_setting(optimization_setting):
             return
@@ -535,6 +548,7 @@ class BacktestingEngine:
             evaluate_func,
             optimization_setting,
             get_target_value,
+            max_workers=max_workers,
             output=self.output
         )
 
@@ -547,7 +561,12 @@ class BacktestingEngine:
 
     run_optimization = run_bf_optimization
 
-    def run_ga_optimization(self, optimization_setting: OptimizationSetting, output=True) -> list:
+    def run_ga_optimization(
+        self,
+        optimization_setting: OptimizationSetting,
+        output: bool = True,
+        max_workers: int = None
+    ) -> list:
         """"""
         if not check_optimization_setting(optimization_setting):
             return
@@ -557,6 +576,7 @@ class BacktestingEngine:
             evaluate_func,
             optimization_setting,
             get_target_value,
+            max_workers=max_workers,
             output=self.output
         )
 
@@ -922,6 +942,12 @@ class BacktestingEngine:
         """
         return self.pricetick
 
+    def get_size(self, strategy: CtaTemplate) -> int:
+        """
+        Return contract size data.
+        """
+        return self.size
+
     def put_strategy_event(self, strategy: CtaTemplate) -> None:
         """
         Put an event to update strategy status.
@@ -987,8 +1013,7 @@ class DailyResult:
         start_pos: float,
         size: int,
         rate: float,
-        slippage: float,
-        inverse: bool
+        slippage: float
     ) -> None:
         """"""
         # If no pre_close provided on the first day,
@@ -1002,12 +1027,7 @@ class DailyResult:
         self.start_pos = start_pos
         self.end_pos = start_pos
 
-        if not inverse:     # For normal contract
-            self.holding_pnl = self.start_pos * \
-                (self.close_price - self.pre_close) * size
-        else:               # For inverse contract
-            self.holding_pnl = self.start_pos * \
-                (1 / self.pre_close - 1 / self.close_price) * size
+        self.holding_pnl = self.start_pos * (self.close_price - self.pre_close) * size
 
         # Trading pnl is the pnl from new trade during the day
         self.trade_count = len(self.trades)
@@ -1020,18 +1040,10 @@ class DailyResult:
 
             self.end_pos += pos_change
 
-            # For normal contract
-            if not inverse:
-                turnover: float = trade.volume * size * trade.price
-                self.trading_pnl += pos_change * \
-                    (self.close_price - trade.price) * size
-                self.slippage += trade.volume * size * slippage
-            # For inverse contract
-            else:
-                turnover: float = trade.volume * size / trade.price
-                self.trading_pnl += pos_change * \
-                    (1 / trade.price - 1 / self.close_price) * size
-                self.slippage += trade.volume * size * slippage / (trade.price ** 2)
+            turnover: float = trade.volume * size * trade.price
+            self.trading_pnl += pos_change * \
+                (self.close_price - trade.price) * size
+            self.slippage += trade.volume * size * slippage
 
             self.turnover += turnover
             self.commission += turnover * rate
@@ -1085,7 +1097,6 @@ def evaluate(
     capital: int,
     end: datetime,
     mode: BacktestingMode,
-    inverse: bool,
     setting: dict
 ) -> tuple:
     """
@@ -1103,8 +1114,7 @@ def evaluate(
         pricetick=pricetick,
         capital=capital,
         end=end,
-        mode=mode,
-        inverse=inverse
+        mode=mode
     )
 
     engine.add_strategy(strategy_class, setting)
@@ -1134,8 +1144,7 @@ def wrap_evaluate(engine: BacktestingEngine, target_name: str) -> callable:
         engine.pricetick,
         engine.capital,
         engine.end,
-        engine.mode,
-        engine.inverse
+        engine.mode
     )
     return func
 
